@@ -1,5 +1,5 @@
 package OPM::Maker::Command::sopm;
-$OPM::Maker::Command::sopm::VERSION = '1.0.0';
+$OPM::Maker::Command::sopm::VERSION = '1.1.0';
 use v5.10;
 
 use strict;
@@ -333,7 +333,8 @@ sub execute {
     </Database$action_type>~, join "\n", @{ $db_actions{$action_type}->{$phase} };
         }
     }
-    
+
+    CODE:
     for my $code ( @{ $json->{code} || [] } ) {
         if ( !ref $code ) {
             $code = {
@@ -344,6 +345,12 @@ sub execute {
         }
 
         $code->{type} = 'Code' . $code->{type};
+
+        if ( $code->{inline} ) {
+            push @xml_parts, _InlineCode( $code );
+            next CODE;
+        }
+
         push @xml_parts, $utils->packagesetup(
             $code->{type},
             $code->{version},
@@ -391,6 +398,34 @@ sub execute {
     $fh->close;
 }
 
+sub _InlineCode {
+    my ($code) = @_;
+
+    my @parts = split /::/, $code->{inline};
+
+    my $method  = pop @parts;
+    $parts[-1] .= '.pm';
+    my $file    = Path::Class::File->new( @parts );
+
+    my $content = $file->slurp( iomode => '<:encoding(utf-8)' );
+
+    my ($method_body) = $content =~ m{
+        ^sub \s+ \Q$method\E \s* \{ \s+
+            (.*?)
+        ^\}\s+
+    }xms;
+
+    my $version = $code->{version} ?
+        ' Version="' . $code->{version} . '"' :
+        '';
+
+    my $xml = sprintf q~    <%s Type="%s"%s><![CDATA[
+        %s
+    ]]></%s>~, $code->{type}, $code->{phase} // 'post', $version, $method_body, $code->{type};
+
+    return $xml;
+}
+
 sub _IntroTemplate {
     my ($intro) = @_;
 
@@ -700,7 +735,7 @@ OPM::Maker::Command::sopm - Build .sopm file based on metadata
 
 =head1 VERSION
 
-version 1.0.0
+version 1.1.0
 
 =head1 DESCRIPTION
 
@@ -820,6 +855,80 @@ Creates those tags
 
 =head3 Change Column
 
+=head2 Code execution
+
+You can execute code when an addon is installed. As you can see above, there are to 
+phases when code can be executed.
+
+=head3 Run "post" code
+
+Sample config
+
+    "code" : [
+        { "type" : "Install" },
+        { "type" : "Uninstall" }
+    ]
+
+This creates
+
+    <CodeInstall Type="post"><![CDATA[
+        $Kernel::OM->Get('var::packagesetup::' . $Param{Structure}->{Name}->{Content} )->CodeInstall();
+    ]]></CodeInstall>
+    <CodeUninstall Type="post"><![CDATA[
+        $Kernel::OM->Get('var::packagesetup::' . $Param{Structure}->{Name}->{Content} )->CodeUninstall();
+    ]]></CodeUninstall>
+
+You need to provide the C<var::packagesetup::xxxxx> package (where I<xxxxx> is the name of the addon).
+
+There are four types:
+
+=over 4
+
+=item * Install
+
+=item * Upgrade
+
+=item * Reinstall
+
+=item * Uninstall
+
+=back
+
+The methods called are usually named C<CodeInstall>, C<CodeUninstall>, etc. If you need to run an other
+method, you can specify the method to be called:
+
+    "code" : [
+        { "type" : "Install", "function": "OtherMethod" }
+    ]
+
+And that would create
+
+    <CodeInstall Type="post"><![CDATA[
+        $Kernel::OM->Get('var::packagesetup::' . $Param{Structure}->{Name}->{Content} )->OtherMethod();
+    ]]></CodeInstall>
+
+Usually you would run the code after the files
+are installed and database changes are done. But sometime you need to run code before
+this stuff is done. As the packagesetup file isn't available you, you would need to provide
+the Perl code in the JSON file. That's nasty. With I<inline> you can tell this command
+to get the Perl code from a file.
+
+=head3 Options
+
+=over 4
+
+=item * type
+
+=item * version
+
+=item * phase
+
+=item * function
+
+=item * inline
+
+=back
+
 =head1 METHODS
 
 =head2 VERSION
